window.WosaiJSBridge || (function() {
  function warn(msg) {
    // 模板字符串
    console.error("[WosaiJSBridge error]: " + msg);
  }

  function log(msg, object) {
    if (object === undefined) {
      console.log("[WosaiJSBridge log]: " + msg);
    } else {
      console.log("[WosaiJSBridge log]: " + msg, object);
    }
  }

  log("begin load WosaiJSBridge");

  var JSBRIDGE_PROTOCOL = "WosaiJSBridge";
  // 本地注册的方法集合,原生只能调用本地注册的方法,否则会提示错误
  var eventHandlers = {};
  // 短期回调函数集合
  // 在原生调用完对应的方法后,会执行对应的回调函数id，并删除
  var responseCallbacks = {};
  // 长期存在的回调，调用后不会删除
  var responseCallbacksLongTerm = {};
  // 唯一id,用来确保长期回调的唯一性，初始化为最大值
  var uniqueLongCallbackId = 2147483647;

  window.WosaiJSBridge = {
    /**
     * 调用jsbridge api方法
     * @param {String} method 必传，方法名
     * @param {JSON} param 可不传，传递参数
     * @param {Function} callback 可不传，回调函数
     * @param {Boolean} isLongCb 可不传，默认false，是否是长期回调，如果是，则会生成一个长期回调id，以长期回调的形式存在
     */
    call: function(method, param, callback, isLongCb) {
      doSend(method, param, callback, isLongCb);
    },
    /**
     * 调用长期回调jsbridge api方法
     * @param {String} method 必传，方法名
     * @param {JSON} param 可不传，传递参数
     * @param {Function} callback 可不传，回调函数
     */
    callAndKeepAlive: function(method, param, callback) {
      doSend(method, param, callback, true);
    },
    /**
     * 注册事件监听（如H5容器的生命周期）
     * @param {String} eventName 事件名
     * @param {Function} callback 回调函数
     */
    registerEvent: function(eventName, callback) {
      if ('string' !== typeof eventName) {
        return;
      }
      if (!eventHandlers.hasOwnProperty(eventName)) {
        eventHandlers[eventName] = callback;
      } else {
        warn(eventName + ' already exists');
      }
    },
    /**
     * 注册通知事件
     * @param {String} eventName 事件名
     * @param {JSON} param 可不传，传递参数
     * @param {Function} callback 接收回调函数
     */
    registerNotification: function(eventName, param, callback) {
      if ('string' !== typeof eventName) {
        return;
      }

      if (!param && typeof(param) != 'undefined' && param != 0) {
        param = {};
      } else if ('function' === typeof param) {
        callback = param;
        param = {};
      } else if (typeof param !== 'object') {
        param = {};
      }

      param.eventName = eventName;
      doSend('sRegisterNotification', param, callback, true);
    },
    /**
     * 发送通知事件
     * @param {String} eventName 事件名
     * @param {JSON} param 可不传，传递参数
     * @param {Function} callback 回调函数
     */
    sendNotification: function(eventName, param, callback) {
      if ('string' !== typeof eventName) {
        return;
      }

      if (!param && typeof(param) != 'undefined' && param != 0) {
        param = {};
      } else if ('function' === typeof param) {
        callback = param;
        param = {};
      } else if (typeof param !== 'object') {
        param = {};
      }

      var params = {
        eventName: eventName,
        data: param
      };
      doSend('sSendNotification', params, callback, false);
    },
    /**
     * 移除通知事件
     * @param {String} eventName 事件名
     * @param {JSON} param 可不传，传递参数
     * @param {Function} callback 回调函数
     */
    removeNotification: function(eventName, param, callback) {
      if ('string' !== typeof eventName) {
        return;
      }

      if (!param && typeof(param) != 'undefined' && param != 0) {
        param = {};
      } else if ('function' === typeof param) {
        callback = param;
        param = {};
      } else if (typeof param !== 'object') {
        param = {};
      }

      param.eventName = eventName;
      doSend('sRemoveNotification', param, callback, false);
    },
    /**
     * jsbridge回调方法
     * @param {String} responseId 事件名
     * @param {JSON} responseData 传递参数
     */
    handleMessageFromNative: function(responseId, responseData) {
      log("handle message callback >>> " + responseId, responseData);
      // 这里规定,原生执行方法完毕后准备通知h5执行回调时,回调函数id是responseId
      var responseCallback = responseCallbacks[responseId];
      // 默认先短期再长期
      responseCallback = responseCallback || responseCallbacksLongTerm[responseId];
      // 执行本地的回调函数
      responseCallback && responseCallback(responseData);
      delete responseCallbacks[responseId];
    },
    /**
     * 注册事件回调方法（如H5容器的生命周期回调）
     * @param {String} eventName 事件名
     * @param {JSON} responseData 传递参数
     */
    handleEventFromNative: function(eventName, responseData) {
      log("handle event callback >>> " + eventName, responseData);
      /**
       * 否则,代表原生主动执行h5本地的函数
       * 从本地注册的函数中获取
       */
      var handler = eventHandlers[eventName];
      log("handle event >>> " + eventName, handler);
      // 执行本地函数,按照要求传入数据和回调
      handler && handler(responseData);
    }
  };

  /**
   * JS调用原生方法前,会先send到这里进行处理
   * 有三大类型，短期回调，延时回调，长期回调，其中长期回调中又有一个event比较特殊
   * method 方法名
   * param {JSON} options 配置参数，包括
   * data 额外参数
   * isLongCb 是否是长期回调，如果是，则会生成一个长期回调id，以长期回调的形式存在
   * proto 对应方法的模块名
   * 其它 代表相应的头部
   */
  function doSend(method, param, callback, isLongCb) {
    if ('string' !== typeof method) {
      return;
    }

    if (!param && typeof(param) != 'undefined' && param != 0) {
      param = {};
    } else if ('function' === typeof param) {
      if ('boolean' === typeof callback) {
        isLongCb = callback
      }
      callback = param;
      param = {};
    } else if (typeof param !== 'object') {
      param = {};
    }

    var callbackId = -1;
    if ('function' === typeof callback) {
      if (isLongCb) {
        callbackId = Util.getLongCallbackId();
        responseCallbacksLongTerm[callbackId] = callback;
      } else {
        callbackId = Util.getCallbackId();
        responseCallbacks[callbackId] = callback;
      }
    }
    param.callStartTime = new Date().getTime();
    var uri = Util.getUri("callback", method, param, callbackId);
    log(uri);
    if (os.wosai) {
      // 依赖于os判断
      if (os.ios) {
        // ios采用 WosaiJSBridge://callback:1234/method?param
        window.webkit.messageHandlers.WosaiJSBridge.postMessage(uri);
      } else {
        window.top.prompt(uri, "");
      }
    } else {
      // 浏览器
      warn(
        "\u6D4F\u89C8\u5668\u4E2Djsbridge\u65E0\u6548，\u5BF9\u5E94scheme: " + uri);
    }
  }

  var Util = {
    /**
     * 获取短期回调id，内部要避免和长期回调的冲突
     * @return {Number} 返回一个随机的短期回调id
     */
    getCallbackId: function() {
      //随机生成callbackId
      return Math.floor(Math.random() * uniqueLongCallbackId);
      //return '' + new Date().getTime() + (Math.random());
    },
    /**
     * 获得本地的长期回调，每一次都是一个唯一的值
     * @retrurn 返回对应的回调id
     * @return {Number} 返回长期回调id
     */
    getLongCallbackId: function() {
      uniqueLongCallbackId -= 1;
      return uniqueLongCallbackId;
    },
    getUri: function(moduleName, method, param, callbackId) {
      //生成native需要的协议uri
      param = this.getParam(param);
      var uri =
        JSBRIDGE_PROTOCOL +
        "://" +
        moduleName +
        ":" +
        callbackId +
        "/" +
        method +
        "?" +
        param;
      return uri;
    },
    /**
     * 将JSON参数转为字符串
     * @param {Object} data 对应的json对象
     * @return {String} 转为字符串后的结果
     */
    getParam: function(obj) {
      //生成json字符串
      if (obj && typeof obj === "object") {
        try {
          return JSON.stringify(obj);
        } catch (e) {
          console.log(e.name + ": " + e.message);
          return "{}";
        }
      } else {
        return obj || "{}"
      }
    }
  };

  function detect(ua) {
    this.os = {};

    var android = ua.match(/(Android);?[\s/]+([\d.]+)?/);

    if (android) {
      this.os.android = true;
      this.os.version = android[2];
      this.os.isBadAndroid = !/Chrome\/\d/.test(window.navigator.appVersion);
    }

    var iphone = ua.match(/(iPhone\sOS)\s([\d_]+)/);

    if (iphone) {
      this.os.ios = true;
      this.os.iphone = true;
      this.os.version = iphone[2].replace(/_/g, ".");
    }

    var ipad = ua.match(/(iPad).*OS\s([\d_]+)/);

    if (ipad) {
      this.os.ios = true;
      this.os.ipad = true;
      this.os.version = ipad[2].replace(/_/g, ".");
    }

    var wosai = ua.match(/WosaiWebView/i);

    if (wosai) {
      this.os.wosai = true;
    }

    // 如果不是wosai，则默认为h5
    if (!wosai) {
      this.os.h5 = true;
    }
  }

  detect(window.navigator.userAgent);

  var readyEvent = document.createEvent('Events');
  readyEvent.initEvent('WosaiJSBridgeReady', !1, !1);
  var docAddEventListener = document.addEventListener;
  document.addEventListener = function(name, func) {
    if (name === readyEvent.type) {
      setTimeout(function() {
        func(readyEvent)
      }, 1);
    }
    docAddEventListener.apply(document, arguments);
  };
  document.dispatchEvent(readyEvent);
  log("load WosaiJSBridge dispatchEvent WosaiJSBridgeReady", WosaiJSBridge);

  log("load WosaiJSBridge finish", WosaiJSBridge);
})();